package query

import (
	"github.com/odysseus/service-registry/common"
	"github.com/odysseus/service-registry/registry"
	"github.com/redis/go-redis/v9"
	"sync"
)

type ServiceQuerier struct {
	rdb      *redis.Client
	queriers map[common.ServiceType]ServiceQuery
	mux      sync.RWMutex
}

var qs = make(map[common.ServiceType]func(rdb *redis.Client) ServiceQuery)

func init() {
	qs[common.SERVICE_NODE_MANAGER] = getNodeManagerQuery
	qs[common.SERVICE_API_GATEWAY] = getGatewayQuery
	qs[common.SERVICE_BACKEND] = getBackendQuery
	qs[common.SERVICE_SCHEDULER] = getSchedulerQuery
	qs[common.SERVICE_WORKER] = getWorkerQuery
}

func NewQuery(redisParam registry.RedisConnParam) *ServiceQuerier {
	rdb := redis.NewClient(&redis.Options{
		Addr:     redisParam.Addr,
		Password: redisParam.Password,
		DB:       redisParam.DbIndex,
	})
	queriers := make(map[common.ServiceType]ServiceQuery)
	for k, v := range qs {
		queriers[k] = v(rdb)
	}

	return &ServiceQuerier{
		rdb:      rdb,
		queriers: queriers,
	}
}

func (s *ServiceQuerier) Select(service common.ServiceType) ServiceQuery {
	s.mux.RLock()
	defer s.mux.RUnlock()
	return s.queriers[service]
}
