package query

import (
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
	"sort"
)

type WorkerInfo struct {
	// worker specific info
	HearBeat       int64    `redis:"heartbeat" json:"heartbeat"`
	ActiveNM       []string `redis:"active_nm" json:"active_nm"`
	MinerAddress   string   `redis:"miner_address" json:"miner_address"`
	BenefitAddress string   `redis:"benefit_address" json:"benefit_address"`
	IP             string   `redis:"ip" json:"ip"`
	Nonce          int64    `redis:"nonce" json:"nonce"`
	TaskList       []string `redis:"task_list" json:"task_list"`
	CpuModel       string   `redis:"cpu_model" json:"cpu_model"`
	CpuCore        int      `redis:"cpu_core" json:"cpu_core"`
	GPUModel       string   `redis:"gpu_model" json:"gpu_model" json:"gpu_model"`
	GPURam         int      `redis:"gpu_ram" json:"gpu_ram"`
	GPUCount       int      `redis:"gpu_count" json:"gpu_count"`
	RamTotal       int64    `redis:"ram_total" json:"ram_total"`
	RamUsed        int64    `redis:"ram_used" json:"ram_used"`
	CreateTime     int64    `redis:"create_time" json:"create_time"`
	BootupTime     int64    `redis:"bootup_time" json:"bootup_time"`
	Workload       int      `redis:"workload" json:"workload"`
	MacAddress     string   `redis:"mac_address" json:"mac_address"`
}

type workerQuery struct {
	baseService
}

func getWorkerQuery(rdb *redis.Client) ServiceQuery {
	return workerQuery{
		baseService{
			rdb:     rdb,
			service: common.SERVICE_WORKER,
		},
	}
}

func (g workerQuery) List() ([]string, error) {
	all, err := getAll(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	sort.Sort(ServiceInfoList(all))
	var res []string
	for _, v := range all {
		res = append(res, string(v.Message()))
	}
	return res, nil
}

func (g workerQuery) ListByPage(pageSize int, pageNum int) ([]string, int, error) {
	all, err := getAll(g.rdb, g.service)
	if err != nil {
		return nil, 0, err
	}
	sort.Sort(ServiceInfoList(all))
	start := pageSize * pageNum
	end := start + pageSize
	if len(all) < end {
		end = len(all)
	}
	var res []string
	for _, v := range all[start:end] {
		res = append(res, string(v.Message()))
	}
	return res, len(all), nil
}
